// g++-mp-4.7 -O2 -msse2 -DNDEBUG demo_centroid.cpp -o demo_centroid && ./demo_centroid

/**
 * This demo demonstrates the vectorization performance of
 * Structure of Array versus Array of Structure.
 * 
 * A set of points is stored as a 3xN matrix, and the goal is
 * to compute the centroid of this set of points.
 * 
 * As pointed out below, switch between RowMajor and ColMajor
 * and compare the performance.
 */

#include <Eigen/Geometry>
#include <bench/BenchTimer.h>
#include <iostream>
using namespace Eigen;

const int D = 3;
/// Switch between RowMajor and ColMajor and compare the performance:
typedef Matrix<float,D,Dynamic,RowMajor>  Points;
typedef Matrix<float,D,1>                 Vec;

EIGEN_DONT_INLINE Vec centroid(const Points& points)
{
  return points.rowwise().mean();
}

int main()
{
  int tries = 10;
  int rep = 10000;
  int n = 1000;
  BenchTimer t;
  {
    Points points(D,n);
    points.setRandom();
    BENCH(t, tries, rep, centroid(points));
    std::cout << "running  time: " << t.best() << "s\n";
  }
}

